@@ -4,7 +4,10 @@ chklogical <- function(x){
   ##
   name <- deparse(substitute(x))
   ##
-  if (length(x)!= 1 || !is.logical(x))
+  if (is.numeric(x))
+    x <- as.logical(x)
+  ##
+  if (length(x)!= 1 || !is.logical(x) || is.na(x))
     stop("Argument '", name, "' must be a logical.", call.=FALSE)
   ##
   invisible(NULL)
