@@ -36,6 +36,7 @@ update.meta <- function(object,
                         pooledvar=object$pooledvar,
                         method.smd=object$method.smd,
                         sd.glass=object$sd.glass,
+                        exact.smd=object$exact.smd,
                         method.ci=object$method.ci,
                         byvar=object$byvar,
                         bylab=object$bylab,
@@ -318,7 +319,7 @@ update.meta <- function(object,
                   data=data, subset=subset,
                   ##
                   sm=sm, pooledvar=pooledvar,
-                  method.smd=method.smd, sd.glass=sd.glass,
+                  method.smd=method.smd, sd.glass=sd.glass, exact.smd=exact.smd,
                   ##
                   level=level, level.comb=level.comb,
                   comb.fixed=comb.fixed, comb.random=comb.random,
